import abc

import pbpstats
from pbpstats.resources.enhanced_pbp.foul import Foul


class FreeThrow(metaclass=abc.ABCMeta):
    event_type = 3
    shot_type = pbpstats.FREE_THROW_STRING

    @abc.abstractproperty
    def made(self):
        pass

    @property
    def ft_1_of_1(self):
        return self.event_action_type == 10

    @property
    def ft_1_of_2(self):
        return self.event_action_type == 11

    @property
    def ft_2_of_2(self):
        return self.event_action_type == 12

    @property
    def ft_1_of_3(self):
        return self.event_action_type == 13

    @property
    def ft_2_of_3(self):
        return self.event_action_type == 14

    @property
    def ft_3_of_3(self):
        return self.event_action_type == 15

    @property
    def first_ft(self):
        return '1 of' in self.description or self.ft_1pt or self.ft_2pt or self.ft_3pt

    @property
    def end_ft(self):
        return self.ft_1_of_1 or self.ft_2_of_2 or self.ft_3_of_3 or self.ft_1pt or self.ft_2pt or self.ft_3pt

    @property
    def technical_ft(self):
        return ' Technical' in self.description

    @property
    def ft_1pt(self):
        """
        only used in g-league, starting in 2019-20 season
        """
        return self.event_action_type == 30 or self.event_action_type == 35

    @property
    def ft_2pt(self):
        """
        only used in g-league, starting in 2019-20 season
        """
        return self.event_action_type == 31 or self.event_action_type == 36

    @property
    def ft_3pt(self):
        """
        only used in g-league, starting in 2019-20 season
        """
        return self.event_action_type == 32 or self.event_action_type == 37

    @property
    def shot_value(self):
        if self.ft_2pt:
            return 2
        if self.ft_3pt:
            return 3
        return 1

    @property
    def is_away_from_play_ft(self):
        foul = self.foul_that_led_to_ft
        if ((self.ft_1_of_1 or self.ft_1pt) or (self.ft_2_of_2 or self.ft_2pt)) and foul is not None and foul.is_away_from_play_foul:
            made_shots_at_event_time = []
            fts_by_other_player_at_event_time = []
            events_at_event_time = self.get_all_events_at_current_time()
            for event in events_at_event_time:
                if hasattr(event, 'made') and event.made:
                    if not isinstance(event, FreeThrow):
                        made_shots_at_event_time.append(event)
                    if isinstance(event, FreeThrow) and event.player1_id != self.player1_id:
                        fts_by_other_player_at_event_time.append(event)

            if len(made_shots_at_event_time) == 0:
                # check for made free throw by other player - this is where player is fouled going for rebound on made FT
                if len(fts_by_other_player_at_event_time) == 0:
                    return True
                for ft in fts_by_other_player_at_event_time:
                    if ft.team_id != self.team_id:
                        return True
            else:
                made_shots_at_event_time = sorted(made_shots_at_event_time, key=lambda k: k.order)
                if (made_shots_at_event_time[0].team_id == foul.team_id) and (self.player1_id != made_shots_at_event_time[0].player1_id):
                    # make sure player who made shot is not player who shot FT
                    return True

        return False

    @property
    def is_inbound_foul_ft(self):
        if self.ft_1_of_1 or self.ft_1pt:
            events_at_event_time = self.get_all_events_at_current_time()
            for event in events_at_event_time:
                if isinstance(event, Foul) and event.is_inbound_foul:
                    return True

        return False

    @property
    def foul_that_led_to_ft(self):
        clock = self.clock
        # foul should be before FT so start by going backwards
        event = self
        while event is not None and event.clock == clock and not (isinstance(event, Foul) and not event.is_technical and not event.is_double_technical):
            event = event.previous_event

        if isinstance(event, Foul) and not event.is_technical and not event.is_double_technical and event.clock == clock:
            return event

        # bug in pbp where foul is after FT
        event = self
        while event is not None and event.clock == clock and not (isinstance(event, Foul) and not event.is_technical and not event.is_double_technical):
            event = event.next_event

        if isinstance(event, Foul) and not event.is_technical and not event.is_double_technical and event.clock == clock:
            return event
        return None

    @property
    def num_ft_for_trip(self):
        if 'of 1' in self.description:
            return 1
        elif 'of 2' in self.description:
            return 2
        elif 'of 3' in self.description:
            return 3

    @property
    def free_throw_type(self):
        if self.technical_ft:
            return 'Technical'
        num_fts = self.num_ft_for_trip

        if num_fts == 1:
            # check for shot before FT at same time as FT
            previous_event = self
            while previous_event is not None and previous_event.clock == self.clock and not (hasattr(previous_event, 'made') and previous_event.made and not isinstance(previous_event, FreeThrow)):
                previous_event = previous_event.previous_event
            if previous_event is not None and previous_event.clock == self.clock and (hasattr(previous_event, 'made') and previous_event.made and not isinstance(previous_event, FreeThrow)):
                and1_shot = previous_event
                if self.player1_id == and1_shot.player1_id:
                    return f'{and1_shot.shot_value}pt And 1'
                else:
                    return '1 Shot Away From Play'
            else:
                return '1 Shot Away From Play'
        foul_event = self.foul_that_led_to_ft
        if foul_event.is_shooting_foul or foul_event.is_shooting_block_foul:
            return f'{num_fts}pt Shooting Foul'
        elif foul_event.is_flagrant:
            if num_fts is None:
                # assume 2 shot flagrant if num_fts is None
                num_fts = 2
            return f'{num_fts} Shot Flagrant'
        elif foul_event.is_away_from_play_foul:
            return f'{num_fts} Shot Away From Play'
        elif foul_event.is_inbound_foul:
            return f'{num_fts} Shot Inbound Foul'
        elif foul_event.is_clear_path_foul:
            return f'{num_fts} Shot Clear Path'
        elif num_fts == 3:
            return '3pt Shooting Foul'
        else:
            return 'Penalty'

    @property
    def event_for_efficiency_stats(self):
        clock = self.clock
        # foul should be before FT so start by going backwards
        event = self
        while event is not None and event.clock == clock and not isinstance(event, Foul):
            event = event.previous_event

        if isinstance(event, Foul) and event.clock == clock:
            return event

        # bug in pbp where foul is after FT
        event = self
        while event is not None and event.clock == clock and not isinstance(event, Foul):
            event = event.next_event

        if isinstance(event, Foul) and event.clock == clock:
            return event
        return self

    @property
    def event_stats(self):
        stats = []
        team_ids = list(self.current_players.keys())
        is_penalty_event = self.is_penalty_event()
        is_second_chance_event = self.is_second_chance_event()
        lineup_ids = self.event_for_efficiency_stats.lineup_ids
        if self.made:
            if self.ft_3pt:
                free_throw_key = pbpstats.FT_3_PT_MADE_STRING
            elif self.ft_2pt:
                free_throw_key = pbpstats.FT_2_PT_MADE_STRING
            elif self.ft_1pt:
                free_throw_key = pbpstats.FT_1_PT_MADE_STRING
            elif self.technical_ft:
                free_throw_key = pbpstats.TECHNICAL_FTS_MADE_STRING
            else:
                free_throw_key = pbpstats.FTS_MADE_STRING
            points = self.shot_value
            stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': free_throw_key, 'stat_value': 1})
            if is_second_chance_event:
                second_chance_stat_key = f'{pbpstats.SECOND_CHANCE_STRING}{free_throw_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': second_chance_stat_key, 'stat_value': 1})
            if is_penalty_event:
                penalty_stat_key = f'{pbpstats.PENALTY_STRING}{free_throw_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': penalty_stat_key, 'stat_value': 1})
                foul_event = self.foul_that_led_to_ft
                if foul_event is not None and foul_event.is_personal_take_foul and self.seconds_remaining < 60 and self.period >= 4:
                    final_minute_take_foul_stat_key = f'{pbpstats.FINAL_MINUTE_PENALTY_TAKE_FOUL_STRING}{free_throw_key}'
                    stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': final_minute_take_foul_stat_key, 'stat_value': 1})

            # add plus minus and opponent points - used for lineup/wowy stats to get net rating
            for team_id, players in self.event_for_efficiency_stats.current_players.items():
                multiplier = 1 if team_id == self.team_id else -1
                opponent_team_id = team_ids[0] if team_id == team_ids[1] else team_ids[1]
                for player_id in players:
                    stat_item = {
                        'player_id': player_id,
                        'team_id': team_id,
                        'opponent_team_id': opponent_team_id,
                        'lineup_id': lineup_ids[team_id],
                        'opponent_lineup_id': lineup_ids[opponent_team_id],
                        'stat_key': pbpstats.PLUS_MINUS_STRING,
                        'stat_value': points * multiplier,
                    }
                    stats.append(stat_item)
                    if multiplier == -1:
                        opponent_points_stat_item = {
                            'player_id': player_id,
                            'team_id': team_id,
                            'opponent_team_id': opponent_team_id,
                            'lineup_id': lineup_ids[team_id],
                            'opponent_lineup_id': lineup_ids[opponent_team_id],
                            'stat_key': pbpstats.OPPONENT_POINTS,
                            'stat_value': points,
                        }
                        stats.append(opponent_points_stat_item)
        if self.first_ft or self.technical_ft:
            free_throw_trip_key = self.free_throw_type + ' Free Throw Trips'
            stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': free_throw_trip_key, 'stat_value': 1})
            if is_second_chance_event:
                second_chance_stat_key = f'{pbpstats.SECOND_CHANCE_STRING}{free_throw_trip_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': second_chance_stat_key, 'stat_value': 1})
            if is_penalty_event:
                penalty_stat_key = f'{pbpstats.PENALTY_STRING}{free_throw_trip_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': penalty_stat_key, 'stat_value': 1})
                foul_event = self.foul_that_led_to_ft
                if foul_event is not None and foul_event.is_personal_take_foul and self.seconds_remaining < 60 and self.period >= 4:
                    current_players = self.event_for_efficiency_stats.current_players
                    offense_team_id = self.get_offense_team_id()
                    for team_id, players in current_players.items():
                        final_minute_take_foul_possessions_stat_key = (
                            f'{pbpstats.FINAL_MINUTE_PENALTY_TAKE_FOUL_STRING}{pbpstats.OFFENSIVE_POSSESSION_STRING}'
                            if team_id == offense_team_id
                            else f'{pbpstats.FINAL_MINUTE_PENALTY_TAKE_FOUL_STRING}{pbpstats.DEFENSIVE_POSSESSION_STRING}'
                        )
                        for player_id in players:
                            stat_item = {
                                'player_id': player_id,
                                'team_id': team_id,
                                'stat_key': final_minute_take_foul_possessions_stat_key,
                                'stat_value': 1,
                            }
                            stats.append(stat_item)

        if not self.made:
            if self.ft_3pt:
                free_throw_key = pbpstats.FT_3_PT_MISSED_STRING
            elif self.ft_2pt:
                free_throw_key = pbpstats.FT_2_PT_MISSED_STRING
            elif self.ft_1pt:
                free_throw_key = pbpstats.FT_1_PT_MISSED_STRING
            else:
                free_throw_key = pbpstats.FTS_MISSED_STRING
            stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': free_throw_key, 'stat_value': 1})
            if is_second_chance_event:
                second_chance_stat_key = f'{pbpstats.SECOND_CHANCE_STRING}{free_throw_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': second_chance_stat_key, 'stat_value': 1})
            if is_penalty_event:
                penalty_stat_key = f'{pbpstats.PENALTY_STRING}{free_throw_key}'
                stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': penalty_stat_key, 'stat_value': 1})
                foul_event = self.foul_that_led_to_ft
                if foul_event is not None and foul_event.is_personal_take_foul and self.seconds_remaining < 60 and self.period >= 4:
                    final_minute_take_foul_stat_key = f'{pbpstats.FINAL_MINUTE_PENALTY_TAKE_FOUL_STRING}{free_throw_key}'
                    stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': final_minute_take_foul_stat_key, 'stat_value': 1})

        opponent_team_id = team_ids[0] if self.team_id == team_ids[1] else team_ids[1]
        for stat in stats:
            if 'lineup_id' not in stat.keys():
                opponent_team_id = team_ids[0] if stat['team_id'] == team_ids[1] else team_ids[1]
                stat['lineup_id'] = lineup_ids[stat['team_id']]
                stat['opponent_team_id'] = opponent_team_id
                stat['opponent_lineup_id'] = lineup_ids[opponent_team_id]

        return self.base_stats + stats



import json
import os

from pbpstats import (
    NBA_STRING, G_LEAGUE_STRING, WNBA_STRING, NBA_GAME_ID_PREFIX,
    G_LEAGUE_GAME_ID_PREFIX, WNBA_GAME_ID_PREFIX
)
from pbpstats.data_loader.stats_nba.file_loader import StatsNbaFileLoader
from pbpstats.data_loader.stats_nba.web_loader import StatsNbaWebLoader
from pbpstats.resources.games.stats_nba_game_item import StatsNbaGameItem


class StatsNbaGameFinderLoader(StatsNbaFileLoader, StatsNbaWebLoader):
    """
    Class for loading game finder data from stats.nba.com
    data.nba.com schedule end point doesn't have data for older season
    this is an easy way to get all games for a season
    """
    data_provider = 'stats_nba'
    resource = 'Games'
    parent_object = 'Season'

    def __init__(self, league, season, season_type, source, file_directory=None):
        self.league_string = league
        self.season_string = season
        self.season_type_string = season_type
        self.file_directory = file_directory
        self.source = source
        self._load_data()
        self._make_game_data_items()

    def _load_data(self):
        source_method = getattr(self, f'_from_{self.source}')
        source_method()

    def _from_file(self):
        self.file_path = f'{self.file_directory}/schedule/stats_{self.league_string}_{self.season_string.replace("-", "_")}_{self.season_type_string.replace(" ", "_")}.json'
        self._load_data_from_file()

    def _from_web(self):
        league_url_part = f'{G_LEAGUE_STRING}.{NBA_STRING}' if self.league_string == G_LEAGUE_STRING else self.league_string
        self.base_url = f'https://stats.{league_url_part}.com/stats/leaguegamefinder'
        self.parameters = {
            "PlayerOrTeam": 'T',
            "gtPTS": 1,
            "Season": self.season_string,
            "SeasonType": self.season_type_string,
            "LeagueID": self.league_id
        }
        self._load_request_data()

    def _save_data_to_file(self):
        if self.file_directory is not None and os.path.isdir(self.file_directory):
            file_path = f'{self.file_directory}/schedule/stats_{self.league_string}_{self.season_string.replace("-", "_")}_{self.season_type_string.replace(" ", "_")}.json'
            with open(file_path, 'w') as outfile:
                json.dump(self.source_data, outfile)

    @property
    def league_id(self):
        if self.league_string == NBA_STRING:
            return NBA_GAME_ID_PREFIX
        elif self.league_string == WNBA_STRING:
            return WNBA_GAME_ID_PREFIX
        elif self.league_string == G_LEAGUE_STRING:
            return G_LEAGUE_GAME_ID_PREFIX

    def _make_game_data_items(self):
        self.items = []
        sorted_games = sorted(self.data, key=lambda k: k['GAME_ID'])
        for team1, team2 in zip(sorted_games[0::2], sorted_games[1::2]):
            item = self._make_single_dict_for_game(team1, team2)
            self.items.append(StatsNbaGameItem(item))

    @staticmethod
    def _make_single_dict_for_game(team1, team2):
        """
        MATCHUP is vistor team @ home team for visitor
        MATCHUP is home team vs visitor team for home
        """
        item = {
            'GAME_ID': team1['GAME_ID'],
            'GAME_DATE_EST': team1['GAME_DATE'],
            'GAME_STATUS_TEXT': 'Final',
            'HOME_TEAM_ID': team1['TEAM_ID'] if '@' not in team1['MATCHUP'] else team2['TEAM_ID'],
            'VISITOR_TEAM_ID': team1['TEAM_ID'] if '@' in team1['MATCHUP'] else team2['TEAM_ID'],
        }
        return item



import pbpstats
from pbpstats.resources.enhanced_pbp.end_of_period import EndOfPeriod
from pbpstats.resources.enhanced_pbp.field_goal import FieldGoal
from pbpstats.resources.enhanced_pbp.free_throw import FreeThrow
from pbpstats.resources.enhanced_pbp.jump_ball import JumpBall
from pbpstats.resources.enhanced_pbp.replay import Replay
from pbpstats.resources.enhanced_pbp.turnover import Turnover
from pbpstats.resources.enhanced_pbp.substitution import Substitution
from pbpstats.resources.enhanced_pbp.timeout import Timeout


class EventOrderError(Exception):
    pass


class Rebound(object):
    event_type = 4

    @property
    def is_real_rebound(self):
        """
        all missed shots have a rebound in the play-by-play but
        not all of these rebounds should be counted as actual rebounds
        """
        if self.is_placeholder:
            return False

        if self.is_buzzer_beater_placeholder:
            return False

        if self.is_turnover_placeholder:
            return False

        if self.is_non_live_ft_placeholder:
            return False

        if self.is_buzzer_beater_rebound_at_shot_time:
            return False

        return True

    @property
    def is_placeholder(self):
        """
        these are team rebounds on for example missed FT 1 of 2
        """
        return self.event_action_type != 0 and self.player1_id == 0

    @property
    def is_turnover_placeholder(self):
        """
        ignore if shot clock violation or kicked ball turnover at time of team rebound
        these get logged as team rebounds but they aren't actual rebounds, they are turnovers
        """
        events_at_event_time = self.get_all_events_at_current_time()
        for event in events_at_event_time:
            if (isinstance(event, Turnover) and (event.is_shot_clock_violation or event.is_kicked_ball)) and self.player1_id == 0:
                return True
        return False

    @property
    def is_non_live_ft_placeholder(self):
        """
        example: rebound after missed flagrant FT 2 of 2
        """
        if isinstance(self.missed_shot, FreeThrow) and not self.missed_shot.end_ft:
            return True
        return False

    @property
    def is_buzzer_beater_placeholder(self):
        """
        rebounds occur after time has expired but are still logged in play-by-play
        """
        next_event = self.next_event
        if isinstance(next_event, Replay):
            next_event = next_event.next_event
        if (
            (self.clock == '00:00.0' or self.clock == '0:00') and
            self.player1_id == 0 and
            (next_event is not None and isinstance(next_event, EndOfPeriod) or next_event is None)
        ):
            return True
        return False

    @property
    def is_buzzer_beater_rebound_at_shot_time(self):
        """
        sometimes rebound on buzzer beater is given the same time as shot - don't count these
        only don't count if rebound is last event before end of period event, ignoring replay events
        """
        if self.missed_shot.seconds_remaining <= 3 and self.seconds_remaining == self.missed_shot.seconds_remaining and self.player1_id == 0:
            next_event = self.next_event
            if isinstance(next_event, Replay):
                next_event = next_event.next_event
            if isinstance(next_event, EndOfPeriod):
                return True
        return False

    @property
    def missed_shot(self):
        if isinstance(self.previous_event, (FieldGoal, FreeThrow)):
            if not self.previous_event.made:
                return self.previous_event
        elif isinstance(self.previous_event, Turnover) and self.previous_event.is_shot_clock_violation:
            if isinstance(self.previous_event, FieldGoal):
                return self.previous_event.previous_event
        elif isinstance(self.previous_event, JumpBall):
            prev_event = self.previous_event.previous_event
            while isinstance(prev_event, (Substitution, Timeout)):
                prev_event = prev_event.previous_event
            if isinstance(prev_event, (FieldGoal, FreeThrow)):
                return prev_event
        raise EventOrderError(f'previous event: {self.previous_event} is not a missed free throw or field goal')

    @property
    def oreb(self):
        return self.team_id == self.missed_shot.team_id

    @property
    def self_reb(self):
        return self.player1_id == self.missed_shot.player1_id

    @property
    def event_stats(self):
        stats = []
        if self.is_real_rebound:
            shot_type = self.missed_shot.shot_type
            if isinstance(self.missed_shot, FieldGoal) and self.missed_shot.blocked:
                if not self.oreb:
                    blocked_recovered_key = pbpstats.BLOCKED_STRING + shot_type + 'Recovered'
                    block_player_id = self.missed_shot.player3_id
                    stats.append({'player_id': block_player_id, 'team_id': self.team_id, 'stat_key': blocked_recovered_key, 'stat_value': 1})
                shot_type += pbpstats.BLOCKED_STRING
            if self.oreb:
                reb_key = shot_type + pbpstats.OFFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUNDS_STRING
                opportunity_key = shot_type + pbpstats.OFFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUND_OPPORTUNITIES_STRING
                opponent_opportunity_key = shot_type + pbpstats.DEFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUND_OPPORTUNITIES_STRING
            else:
                reb_key = shot_type + pbpstats.DEFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUNDS_STRING
                opportunity_key = shot_type + pbpstats.DEFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUND_OPPORTUNITIES_STRING
                opponent_opportunity_key = shot_type + pbpstats.OFFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUND_OPPORTUNITIES_STRING

            stats.append({'player_id': self.player1_id, 'team_id': self.team_id, 'stat_key': reb_key, 'stat_value': 1})

            # rebound opportunites for all players on floor
            for team_id, players in self.current_players.items():
                stat_key = opportunity_key if team_id == self.team_id else opponent_opportunity_key
                for player_id in players:
                    stat_item = {
                        'player_id': player_id,
                        'team_id': team_id,
                        'stat_key': stat_key,
                        'stat_value': 1,
                    }
                    stats.append(stat_item)
                if team_id == self.team_id and self.oreb:
                    for player_id in players:
                        on_floor_oreb_stat_item = {
                            'player_id': player_id,
                            'team_id': team_id,
                            'stat_key': pbpstats.ON_FLOOR_OFFENSIVE_REBOUND_STRING,
                            'stat_value': 1,
                        }
                        stats.append(on_floor_oreb_stat_item)

            # player missed shot rebound stats
            shooter_player_id = self.missed_shot.player1_id
            missed_shot_rebounded_opportunity_key = shot_type + pbpstats.OFFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUNDED_OPPORTUNITIES_STRING
            stats.append({'player_id': shooter_player_id, 'team_id': self.missed_shot.team_id, 'stat_key': missed_shot_rebounded_opportunity_key, 'stat_value': 1})
            if self.oreb:
                missed_shot_rebounded_key = shot_type + pbpstats.OFFENSIVE_ABBREVIATION_PREFIX + pbpstats.REBOUNDED_STRING
                stats.append({'player_id': shooter_player_id, 'team_id': self.missed_shot.team_id, 'stat_key': missed_shot_rebounded_key, 'stat_value': 1})
                if self.self_reb:
                    self_rebounded_key = shot_type + pbpstats.SELF_REBOUND_STRING
                    stats.append({'player_id': shooter_player_id, 'team_id': self.missed_shot.team_id, 'stat_key': self_rebounded_key, 'stat_value': 1})

            team_ids = list(self.current_players.keys())
            opponent_team_id = team_ids[0] if self.team_id == team_ids[1] else team_ids[1]
            lineups_ids = self.lineup_ids
            for stat in stats:
                opponent_team_id = team_ids[0] if stat['team_id'] == team_ids[1] else team_ids[1]
                stat['lineup_id'] = lineups_ids[stat['team_id']]
                stat['opponent_team_id'] = opponent_team_id
                stat['opponent_lineup_id'] = lineups_ids[opponent_team_id]
        return self.base_stats + stats



import json
import os
import requests
from pathlib import Path

from pbpstats import HEADERS, REQUEST_TIMEOUT, NBA_STRING, G_LEAGUE_STRING
from pbpstats.data_loader.stats_nba.web_loader import StatsNbaWebLoader
from pbpstats.data_loader.stats_nba.file_loader import StatsNbaFileLoader
from pbpstats.data_loader.stats_nba.summary_loader import StatsNbaSummaryLoader
from pbpstats.resources.shots.stats_nba_shot import StatsNbaShot


class StatsNbaShotsLoader(StatsNbaWebLoader, StatsNbaFileLoader):
    data_provider = 'stats_nba'
    resource = 'Shots'
    parent_object = 'Game'

    def __init__(self, game_id, source, file_directory):
        self.game_id = game_id
        self.source = source
        self.file_directory = file_directory
        self._load_data()
        self._make_shot_items()

    def _load_data(self):
        source_method = getattr(self, f'_from_{self.source}')
        source_method()

    def _from_file(self):
        self.home_file_path = f'{self.file_directory }/game_details/stats_home_shots_{self.game_id}.json'
        self.away_file_path = f'{self.file_directory }/game_details/stats_away_shots_{self.game_id}.json'

        home_data_file = Path(self.home_file_path)
        if not home_data_file.is_file():
            raise Exception(f'{self.home_file_path} does not exist')
        with open(self.home_file_path) as json_data:
            self.home_source_data = json.load(json_data)

        away_data_file = Path(self.away_file_path)
        if not away_data_file.is_file():
            raise Exception(f'{self.away_file_path} does not exist')
        with open(self.away_file_path) as json_data:
            self.away_source_data = json.load(json_data)

    def _from_web(self):
        summary = StatsNbaSummaryLoader(self.game_id, self.source, self.file_directory)
        home_team_id = summary.items[0].home_team_id
        away_team_id = summary.items[0].visitor_team_id
        league_url_part = f'{G_LEAGUE_STRING}.{NBA_STRING}' if self.league == G_LEAGUE_STRING else self.league
        self.base_url = f'https://stats.{league_url_part}.com/stats/shotchartdetail'
        base_paramaters = {
            'GameID': self.game_id,
            'Season': self.season,
            'SeasonType': self.season_type,
            'PlayerID': 0,
            'Outcome': '',
            'Location': '',
            'Month': 0,
            'SeasonSegment': '',
            'DateFrom': '',
            'DateTo': '',
            'OpponentTeamID': 0,
            'VsConference': '',
            'VsDivision': '',
            'Position': '',
            'RookieYear': '',
            'GameSegment': '',
            'Period': 0,
            'LastNGames': 0,
            'ContextMeasure': 'FG_PCT',
            'PlayerPosition': '',
            'LeagueID': self.game_id[0:2],
        }
        self.home_parameters = base_paramaters.copy()
        self.home_parameters['TeamID'] = home_team_id
        self.away_parameters = base_paramaters.copy()
        self.away_parameters['TeamID'] = away_team_id

        response = requests.get(self.base_url, self.home_parameters, headers=HEADERS, timeout=REQUEST_TIMEOUT)
        if response.status_code == 200:
            self.home_source_data = response.json()
        else:
            response.raise_for_status()

        response = requests.get(self.base_url, self.away_parameters, headers=HEADERS, timeout=REQUEST_TIMEOUT)
        if response.status_code == 200:
            self.away_source_data = response.json()
        else:
            response.raise_for_status()
        self._save_data_to_file()

    def _save_data_to_file(self):
        if self.file_directory is not None and os.path.isdir(self.file_directory):
            home_file_path = f'{self.file_directory}/game_details/stats_home_shots_{self.game_id}.json'
            with open(home_file_path, 'w') as outfile:
                json.dump(self.home_source_data, outfile)

            away_file_path = f'{self.file_directory}/game_details/stats_away_shots_{self.game_id}.json'
            with open(away_file_path, 'w') as outfile:
                json.dump(self.away_source_data, outfile)

    def _make_shot_items(self):
        self.items = [StatsNbaShot(item) for item in self.data]

    def make_list_of_dicts(self, results_set_index=0):
        """
        creates list of dicts from data
        """
        headers = self.home_source_data['resultSets'][results_set_index]['headers']
        home_rows = self.home_source_data['resultSets'][results_set_index]['rowSet']
        home_deduped_rows = self.dedupe_events_row_set(home_rows)
        away_rows = self.away_source_data['resultSets'][results_set_index]['rowSet']
        away_deduped_rows = self.dedupe_events_row_set(away_rows)
        return [dict(zip(headers, row)) for row in home_deduped_rows] + [dict(zip(headers, row)) for row in away_deduped_rows]
